var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

import React, { forwardRef, useRef, useImperativeHandle, useEffect, useContext, useState } from 'react';
import Instance from './instance.js';
import BScroll from 'better-scroll';
import Context from '../Context/instance.js';

var timeout = null;

var PDFView = forwardRef(function (_ref, ref) {
  var insertPageElements = _ref.insertPageElements,
      pictures = _ref.pictures,
      src = _ref.src,
      cMapUrl = _ref.cMapUrl,
      _ref$params = _ref.params,
      params = _ref$params === undefined ? {} : _ref$params,
      zoom = _ref.zoom,
      _ref$wrapperAttribute = _ref.wrapperAttribute,
      wrapperAttribute = _ref$wrapperAttribute === undefined ? {} : _ref$wrapperAttribute,
      others = _objectWithoutProperties(_ref, ['insertPageElements', 'pictures', 'src', 'cMapUrl', 'params', 'zoom', 'wrapperAttribute']);

  var refEl = useRef(null);
  useImperativeHandle(ref, function () {
    return refEl.current;
  });
  // context
  var context = useContext(Context) || {};
  var locale = context.locale || function (remark) {
    return remark || '';
  };
  // 总页数

  var _useState = useState(0),
      total = _useState[0],
      setTotal = _useState[1];
  // 初始化


  useEffect(function () {
    init();
  }, []); // eslint-disable-line

  // 更新src或者pictures
  function usePrevious(value) {
    var ref = useRef();
    useEffect(function () {
      ref.current = value;
    });
    return ref.current;
  }
  var prevSrc = usePrevious(src);
  var prevPictures = usePrevious(pictures);

  useEffect(function () {
    // 修改PDF原文件, 刷新整个页面, 从第1页开始重新渲染
    if (refEl.current && refEl.current.instance && (src !== prevSrc || JSON.stringify(pictures) !== JSON.stringify(prevPictures))) {
      init();
    }
  }, [src, pictures]); // eslint-disable-line

  // 一页加载完成后回调
  function _onLoad(s) {
    console.log('全部加载完成');
    if (params.onLoad) params.onLoad(s);
    if (zoom) {
      // 若允许放大, 使用better-scroll
      if (refEl.current.bscroll) {
        console.log('加载完成, bscroll刷新');
        refEl.current.bscroll.finishPullUp();
        refEl.current.bscroll.refresh();
        return;
      }
      refEl.current.bscroll = new BScroll('.pdf-container', {
        scrollX: true,
        zoom: {
          start: 1,
          min: 1,
          max: 4
        },
        probeType: 2,
        pullUpLoad: {
          threshold: 10
        }
      });
      // 上拉到底部刷新
      refEl.current.bscroll.on('pullingUp', function () {
        refEl.current.instance.addPages();
      });
    } else {
      // 不允许放大, 则使用原生滚动条
      // 上拉到底部
      if (!s.container.getAttribute('data-scroll')) {
        s.container.addEventListener('scroll', onScroll, false);
        s.container.setAttribute('data-scroll', '1');
      }
    }
  }

  // 初始化PDF
  function init() {
    if (!src && !pictures) {
      console.warn('SeedsUI: PDFView请传入src或者pictures');
      return;
    }
    // 如果设置了插入元素, 则需要通过react Node渲染完成后, 再实例化
    if (insertPageElements && insertPageElements.length && src) {
      new Instance().getPDF(src, {
        success: function success(pdf) {
          total = pdf.total;
          setTotal(total);
          instance(total);
        },
        fail: function fail() {
          instance();
        }
      });
    } else if (insertPageElements && insertPageElements.length && pictures) {
      total = pictures.length;
      setTotal(total);
      instance(total);
    } else {
      instance();
    }
  }
  // 实例化和更新
  function instance(rows) {
    // 更新
    if (refEl.current.instance) {
      // 更新一页的条数和src
      refEl.current.instance.update({
        rows: rows,
        src: src
      });
      return;
    }
    // 实例化
    refEl.current.instance = new Instance(refEl.current, _extends({
      loadHTML: locale('加载中', 'in_loading'),
      errorHTML: locale('文件加载失败', 'hint_file_failed_to_load'),
      nodataHTML: locale('暂无数据', 'no_data')
    }, params, {
      pictures: pictures,
      src: src,
      cMapUrl: cMapUrl,
      rows: rows || params.rows,
      onLoad: function onLoad(s) {
        _onLoad(s);
      }
    }));
  }
  // 当不允许放大缩小使用原生滚动时, 滚动到底部加载下一页
  function onScroll(e) {
    var target = e.target;
    var clientHeight = target.clientHeight;
    var scrollHeight = target.scrollHeight;
    var scrollTop = target === document.body ? document.documentElement.scrollTop : target.scrollTop;
    if (scrollTop + clientHeight >= scrollHeight - 2) {
      // 刷新
      if (timeout) {
        window.clearTimeout(timeout);
      }
      timeout = setTimeout(function () {
        refEl.current.instance.addPages();
      }, 500);
    }
  }
  // 滚动到页数
  function scrollToPage(pageIndex) {
    if (!pageIndex || isNaN(pageIndex) || Number(pageIndex) < 1) return;
    var pages = refEl.current.querySelectorAll('.pdf-page');
    var page = pages[Number(pageIndex) - 1];
    if (!page) return;
    if (refEl.current.bscroll) {
      refEl.current.bscroll.scrollToElement(page);
    } else {
      refEl.current.scrollTop = page.offsetTop;
    }
  }
  if (refEl.current) {
    refEl.current.scrollToPage = scrollToPage;
  }
  // 设置total则不分页
  function getTotalDOM(total) {
    var insertPageElements = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    var pageFeatureClass = arguments[2];

    if (!src && !pictures) {
      return null;
    }
    if (!total) return null;
    var DOM = [];
    for (var i = 1; i <= total; i++) {
      var insertDOM = [];
      for (var _iterator = insertPageElements, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
        var _ref2;

        if (_isArray) {
          if (_i >= _iterator.length) break;
          _ref2 = _iterator[_i++];
        } else {
          _i = _iterator.next();
          if (_i.done) break;
          _ref2 = _i.value;
        }

        var pageEl = _ref2;

        if (pageEl && pageEl.page && Number(pageEl.page) === i && pageEl.element) insertDOM.push(pageEl.element);
      }
      DOM.push(React.createElement(
        'div',
        { key: i, className: 'pdf-page' + (pageFeatureClass ? ' ' + pageFeatureClass : '') },
        React.createElement(
          'div',
          { className: 'pdf-page-draw' },
          React.createElement('canvas', { className: 'pdf-page-canvas' }),
          React.createElement('img', { className: 'pdf-page-img', alt: '' }),
          React.createElement(
            'div',
            { className: 'pdf-page-elements' },
            insertDOM
          )
        ),
        React.createElement('img', { alt: '', className: 'pdf-page-img hide' }),
        React.createElement(
          'div',
          { className: 'pdf-page-load' },
          params.loadHTML !== undefined ? params.loadHTML : locale('加载中', 'in_loading')
        ),
        React.createElement(
          'div',
          { className: 'pdf-page-error hide' },
          params.errorHTML !== undefined ? params.errorHTML : locale('文件加载失败', 'hint_file_failed_to_load')
        ),
        React.createElement(
          'div',
          { className: 'pdf-page-nodata hide' },
          params.nodataHTML !== undefined ? params.nodataHTML : locale('暂无数据', 'no_date')
        )
      ));
    }
    return DOM;
  }

  return React.createElement(
    'div',
    _extends({ ref: refEl }, others, { className: 'pdf-container' + (others.className ? ' ' + others.className : '') + (zoom ? '' : ' scroll') }),
    React.createElement(
      'div',
      _extends({}, wrapperAttribute, { className: 'pdf-wrapper' + (wrapperAttribute.className ? ' ' + wrapperAttribute.className : '') }),
      getTotalDOM(total, insertPageElements, params.pageFeatureClass)
    )
  );
});

export default PDFView;